#! /usr/bin/perl
#
# Scan all subdirectories and create the apropriate index list.
# Scan the arguments and build a proper website from the input.
#
# $Header: /cvsroot/ac-archive/ac-archive-5/gendocs.pl,v 1.7 2003/08/01 21:48:53 guidod Exp $
#
use strict;
my %o; # the options hash
my %out; my $file; # use as $out{$file}


# --------------------------------------------------------------------------
# some options are global - set the defaults here

my $htm = "html";
$htm= "htm" if $ARGV[0] eq "htm";
my $top_builddir=".";
my $topx="macros.html";
my $n = "\n"; my $nn = "<br />\n";

# -------------------------------------------------------------- OPT SCAN ---
# scanning the options and file arguments:
# we differ the input file by its file extension, and derive 
# their output name right here. This is for "macro" and "docus" 
# output type, some other output types will be generated later 
# from information found in these. 
# input structures:
#      @ARGV - the argument list
# output structures
#      $o{$option} ... options found in argument list
#      $out{$file} ... the filename to be generated at last step
#      $out{$file}{orig} the original file name in ARGV
#      $out{$file}{text} the text content of that file
#      $out{$file}{type} either "docus" or "macro"

for (@ARGV)
{
    if (/^--?(\w+)=(.*)/) { $o{$1} = $2; next; }
    if (/^--?no-(\w+)$/) { $o{$1} = ""; next; }
    if (/^--?(\w+)$/) { $o{$1} = "*"; next; }

    if (/[.]htm$/) {
	my $F = $_; $file = $F."l"; $file =~ s/^\w-//;
	open F, "<$F" or next;
	$out{$file}{body} = join ("",<F>);
	$out{$file}{orig} = $F;
	$out{$file}{type} = "docus";
	close F;
    }
    elsif (/[.]m4$/) {
	my $F = $_; $file = $_; $file =~ s:[.]m4$:.$htm:;
	open F, "<$F" or next;
	$out{$file}{text} = join ("", <F>);
	$out{$file}{orig} = $F;
	$F =~ s:[.]m4$::;
	$out{$file}{name} = $F;
	$out{$file}{type} = "macro";
    }
    elsif (/[.]txt$/) {
	my $F = $_; $file = $_; $file =~ s:[.]txt$:.$htm:;
	open F, "<$F" or next;
	$out{$file}{text} = join ("", <F>);
	$out{$file}{orig} = $F;
	$F =~ s:[.]txt$::;
	$out{$file}{name} = $F;
	$out{$file}{type} = "table";
    }
}

# -------------------------------------------------------- MACRO2HTML SUB ---
# the following functions have been borrowed from macro2html.pl

sub escape_html
{
    my $text = $_[0];
    $text =~ s,\&,\&amp\;,g;
    $text =~ s,\",\&quot\;,g;
    $text =~ s,\<,\&lt\;,g;
    $text =~ s,\>,\&gt\;,g;
    return $text;
}

sub escape_blanks
{
    my $text = $_[0];
    $text =~ s,\s,\&nbsp\;,g;
    return $text;
}

sub parse_macro
{
    my $file = $_[0];
    # These are the parts that a macro should contain.
    # my ($synopsis, $version, $author, $copyright, $license);
    # my ($documentation, $macro, $category);

    # Parse the macro's header for the documentation and the
    # keywords;
    # my $pat_comment = '^dnl[ \\t]*(.*)[ \\t]*$';
    # my $pat_verbatim_comment = '^dnl[ \\t][ \\t](.*)[ \\t]*$';
    # my $pat_synopsis = '^dnl[ \\t]+@synopsis[ \\t]+(.*)[ \\t]*$';
    # my $pat_version = '^dnl[ \\t]+@version[ \\t]+(.*)[ \\t]*$';
    # my $pat_author = '^dnl[ \\t]+@author[ \\t]+(.*)[ \\t]*$';
    
    # abbrevs: see autoconf@gnu.org 19.1.2003, here is a new one:
    # @* AX_CFLAGS_WHATEVER (synopsis)
    # @* AX_CXXFLAGS_WHATEVER (alternative)
    # @, Installed Packags, experimental
    # @; the author <is@home>
    # @! license info
    # @(c) copyright info
    # @$Id: gendocs.pl,v 1.7 2003/08/01 21:48:53 guidod Exp $   

    my $last = "";
    for (split (/^/, $out{$file}{text} ))
    {
	chomp; 
	
	if ($last eq "macro")
	{
	    $out{$file}{macro} .= escape_html($_)."\n";
	    next;
	}

	if (m,^dnl\s+\@(?:synopsis|[*])\s+(.*)\s*$,i) # $pat_synopsis
	{
	    $last = "synopsis";
	    $out{$file}{$last} .= "<br>\n" if length $out{$file}{$last};
	    $out{$file}{$last} .= escape_html($1);
	}
	elsif (m,^dnl\s+\@(?:author|[;])\s+(.*)\s*$,i) # $pat_author
	{
	    $last = "author";
	    $out{$file}{$last} = escape_html($1);
	}
	elsif (m,^dnl\s+\@(?:license|[!])\s+(.*)\s*$,i) # license
	{
	    $last = "license";
	    $out{$file}{$last} = escape_html($1);
	}
	elsif (m,^dnl\s+\@(?:copyright|[(][Cc][)])\s+(.*)\s*$,i) # license
	{
	    $last = "copyright";
	    $out{$file}{$last} = escape_html($1);
	}
	elsif (m,^dnl\s+\@version\s+(.*)\s*$,i) # $pat_version
	{
	    $last = "version";
	    my  $line = $1;
	    if ($line =~ m,^\$[I][d]: [^ ]+ ([^ ]+) ([^ ]+),i)
	    {
		$out{$file}{$last} = escape_html($1)." (".escape_html($2).")";
	    }
	    elsif ($line =~ m,^\$[I][d]:(.*)\$(.*),i)
	    {
		$out{$file}{$last} = escape_html($1." ".$2);
	    }else{
		$out{$file}{$last} = escape_html($line);
	    }
	}
	elsif (m,^dnl\s+\@(?:category|[\,])\s+(.*)\s*$,i) # categories
	{
	    $out{$file}{category} = $1; # do not put thru..
	}
	elsif (m,^dnl\s+(obsoleted[:\s].*)\s*$,i) # obsoleted
	{
	    $out{$file}{obsoleted} = $1;
	    $out{$file}{documentation} .= "<br>" if $last eq "text";
	    $out{$file}{documentation} .= "<b>".escape_html($1)."</b>";
	    $out{$file}{documentation} .= "<br>\n";
	    $last = "enum";
	}
	elsif (m,^dnl\s+([*-]\s.*)\s*$,i) # enumeration
	{
	    $out{$file}{documentation} .= "<br>" if $last eq "text";
	    $out{$file}{documentation} .= escape_html($1);
	    $out{$file}{documentation} .= "<br>\n";
	    $last = "enum";
	}
	elsif (m,^dnl\s\s(.*)\s*$,i) # $pat_verbatim_comment
	{
	    $out{$file}{documentation} .= "<br>" if $last eq "text";
	    $out{$file}{documentation} .= "<code>&nbsp;&nbsp;";
	    $out{$file}{documentation} .= escape_blanks(escape_html($1));
	    $out{$file}{documentation} .= "</code><br>\n";
	    $last = "code";
	}
	elsif (m,^dnl\s*(.*)\s*$,i) # $pat_comment
	{
	    if (not length $1) {
		$out{$file}{documentation} .= "<p>";
		$last = "para";
	    }else{
		$out{$file}{documentation} .= escape_html($1)."\n";
		$last = "text";
	    }
	}else{
	    $out{$file}{macro} = escape_html($_)."\n"; # print STDERR ".";
	    # The header part is over. Now the remainder of the file is the
	    # macro source code.
	    $last = "macro";
	}
    }
}

# ------------------------------------- ( Make up the formatted HTML page )
# called for each $file, -> generates $out{$file}{body}.
# the header parts have been already set in parse_macro,
#
# the header parts and body part will be combined into
# the full html output text - originally the full html
# text was made up right here (in $out{$file}{html}) but 
# now we delay the process to the final output loop. IF
# you set the $out{$file}{html} part somewhere in the next
# processing steps THEN the final output loop will prefer
# that one ELSE it creates the output file depending on
# its type merging the $out{$file}{body} part in the middle.
sub create_html                        
{
    my $file = $_[0];

    $out{$file}{archive} = "";
    $out{$file}{archive} = " &nbsp; &nbsp; <small>$o{vers}</small>"
	if length                                 $o{vers};

    my $name = $out{$file}{orig}; $name =~ s:.*/::; $name =~ s:[.]\w+$::;
    my $body = "body";

    if ($body eq "html")
    {
	$out{$file}{$body} = "";
	$out{$file}{$body} .= "<html>"."\n";
	$out{$file}{$body} .= "<head>"."\n";
	$out{$file}{$body} .= "    <title>Macro: ".$name."</title>"."\n";
	$out{$file}{$body} .= "</head>"."\n";
	$out{$file}{$body} .= "<body>"."\n";
    }else{
	$out{$file}{title} = "Macro: ".$name;
    }

    my $_width100=' width="100%"';
    my $_width50=' width="50%"';
    my $_center=' align="center"';
    my $O = '&nbsp;';
    if (length $name > 32)
    {
	$out{$file}{$body} .= "<table$_width100>"."\n";
	$out{$file}{$body} .= "    <tr>"."\n";
	$out{$file}{$body} .= "    <td$_width50$_center><a href=\"../\">"
	    ."Back to the Main Page.</a></td>"."\n";
	$out{$file}{$body} .= "    <td$_width50$_center><a href=\""
	    .$name.".m4\">"
	    ."Download the M4 Source.</a><!--DL--></td>"."\n";
	$out{$file}{$body} .= "    </tr>"."\n";
	$out{$file}{$body} .= "</table>"."\n";
	$out{$file}{$body} .= "<hr><p>"."\n";
    
	$out{$file}{$body} .= "<h1 align=\"center\">".$name."</h1>"."\n";
    }else{
	$out{$file}{$body} .= "<table$_width100>"."\n";
	$out{$file}{$body} .= "    <tr>"."\n";
	$out{$file}{$body} .= "    <td$_center><a href=\"../\">"
	    ."Back${O}to${O}the Main${O}Page.</a></td>"."\n";

	$out{$file}{$body} .= "    <td$_center><h1>". $name."</h1></td>"."\n";

	$out{$file}{$body} .= "    <td$_center><a href=\"".$name.".m4\">"
	    ."Download the${O}M4${O}Source.</a><!--DL--></td>"."\n";
	$out{$file}{$body} .= "    </tr>"."\n";
	$out{$file}{$body} .= "</table>"."\n";
	$out{$file}{$body} .= "<hr><p>"."\n";
    }


    $out{$file}{$body} .= "<dl>\n";
    $out{$file}{$body} .= "<dt><big><b>Synopsis</b></big></dt><dd>".$n;
    $out{$file}{$body} .= " <p><code>".$out{$file}{synopsis}."</code></p>";
    $out{$file}{$body} .= $n."</dd>".$n;
    
    $out{$file}{$body} .= "<dt><big><b>Version</b></big></dt><dd>".$n;
    $out{$file}{$body} .= " <p>".$out{$file}{version};
    $out{$file}{$body} .= $n."".$out{$file}{archive};
    $out{$file}{$body} .= ' &nbsp; : &nbsp; <i>'.$out{$file}{category}."</i>"
	if length                                $out{$file}{category};
    $out{$file}{$body} .= $n."</p>";
    $out{$file}{$body} .= $n."</dd>".$n;
#   $out{$file}{$body} .= "<p><code>".$version."</code>".$n."</dd>";

    $out{$file}{$body} .= "<dt><big><b>Author</b></big></dt><dd>".$n;
    $out{$file}{$body} .= " <p>".$out{$file}{author};
    $out{$file}{$body} .= $n." <br> copyright: ".$out{$file}{copyright}
	if length                             $out{$file}{copyright};

    $out{$file}{$body} .= $n." <br> license: ".$out{$file}{license}
	if length                           $out{$file}{license};

    $out{$file}{$body} .= $n." </p>";
    $out{$file}{$body} .= $n."</dd>".$n;

    $out{$file}{$body} .= $n;
    $out{$file}{$body} .= "<dt><big><b>Description</b></big></dt><dd>".$n;
    $out{$file}{$body} .= " <div>\n".$out{$file}{documentation}."\n</div>";
    $out{$file}{$body} .= $n."</dd>".$n;

    $out{$file}{$body} .= $n.$n;
    $out{$file}{$body} .= "<dt><big><b>M4 Source Code</b></big></dt><dd>".$n;
    $out{$file}{$body} .= " <div><pre>\n".$out{$file}{macro}."\n</pre></div>";
    $out{$file}{$body} .= $n."</dd>".$n;
    $out{$file}{$body} .= "</dl>";

    if ($body eq "html") 
    {
	$out{$file}{$body} .= "</body>"."\n";
	$out{$file}{$body} .= "</html>"."\n";
    }
}

# ------------------------------------------------------------- ASPELL SUB --
# we use an external tool gnu 'aspell' that will probably
# be present on most linux platforms atleast. Note how we
# use a perl pipe as input with the input file name which
# we take interest in to get indexed. The output of `aspell`
# is saved in $out{$file}{spelled} for later evaluation.
# This is just a routine, called from later parts.

sub parse_spell
{
    return if $o{nospell};
    my $file = $_[0];
    my $text = $out{$file}{documentation};
    return if not length $text;
    my %word;
    $text =~ s:\b[A-Z]+[_][A-Z_]*[A-Z]\b::g;
    $text =~ s:<code>[^<>]*</code>: :gs;
    $text =~ s:<[^<>]*>: :g;
    $text =~ s:\&lt\;:<:g;  $text =~ s:\&quot\;:\":g;  
    $text =~ s:\&gt\;:<:g;  $text =~ s:\&amp\;:\&:g;  
    $text =~ s:\&\w+\;: :g;
    $text =~ s{<(\w[^<>]*[.][^<>]+)>}{ $word{$1} = $1; "  "}gmex;
    $text =~ s{(\b\w+[/][\w/]*\w\b)}{ $word{$1} = $1; "  "}gmex;
    $text =~ s{(-l)(\w+\b)}{ $word{"lib".$2} = $1.$2; "  "}gmex;
    $text =~ s{(-L)(\w+\b)}{ $word{"lib-".$2} = $1.$2; "  "}gmex;
    $text =~ s{(--\w[\w-]*\w\b)}{ $word{$1} = $1; "  "}gmex;
    $text =~ s{(-?\w+-[\w-]*\w\b)}{ $word{$1} = $1; "  "}gmex;
    $text =~ s{(-[A-Z])(\w+\b)}{ $word{$1.$2} = $1.$2; "  "}gmex;
    $text =~ s:\=\s?\w+\b: :g;

    my $SPELL = "aspell --language-tag=english list";
    return if not open SPELL, "| $SPELL > $file.spell"; # <--- a pipe!!
    print SPELL $text; 
    close SPELL;
    return if not open SPELL, ">> $file.spell";
    for $text (keys %word) { print SPELL $text,"\n"; }
    close SPELL;
    $out{$file}{spelled} = "$file.spell";
#    $text = $file; $text =~ s:.*/::;
#    my $body; for $body (qw/html body/) 
#    {	if (exists $out{$file}{$body}) 
#	{   $out{$file}{$body} =~ s{<!--DL-->}{
#		"<!--DL-->"."\n"."<a href='$text.spell'>*</a>" }ex; 
#	} 
#    }
}

sub parse_spell_check
{
    return if $o{nospell};
    my $ver = `aspell --version`;
    if ($ver !~ /\d+[.]\d+/) {
	print STDERR "no spell found, $ver";
	$o{nospell} = "*";
    }
}

# -------------------------------------------------------------- DOC SCAN ---
# This is overly complicated and should be changed - It is about
# the "left navigation bar" which is broken into different sections
# each with a set of entries, and these sections/entries do not need
# to follow alphabetic order. Information about that is automatically
# derived from the input files but experiences with other projects
# have shown that it breaks more often than it helps to distribute
# the pieces into each input file. Instead, use a central "nav bar"
# template file that the various docs can attach to as linkends.

for $file (keys %out)
{
    if ($out{$file}{body} =~ /<!--\s*SORTING\s*\"([^\"]+)\"/)
    {
	$out{$file}{sort} = $1 if not length $out{$file}{sort};
    }
    if ($out{$file}{body} =~ /<!--\s*CATEGORY\s*\"([^\"]+)\"/)
    {
	$out{$file}{category} = $1 if not length $out{$file}{category};
	$out{$file}{sort} = $1 if not length $out{$file}{sort};
    }
    if ($out{$file}{body} =~ /<!--\s*CHAPTER\s*\"([^\"]+)\"/)
    {
	$out{$file}{chapter} = $1 if not length $out{$file}{chapter};
    }
    if ($out{$file}{body} =~ /<!--\s*SHORTNAME\s*\"([^\"]+)\"/)
    {
	$out{$file}{title} = $1 if not length $out{$file}{title};
    }
}

# -------------------------------------------------------------- SCAN DIRS --
#
# For each category, find the .m4 macro files in it and list them.
#
$out{$topx}{title} = "The Autoconf Macro Archive!";
my $m4 = "m4";
my $name;

opendir(DIR, $top_builddir) || die("Can't open directory '$top_builddir': $!");
while($name = readdir(DIR))          # open the base directory and
{                                    # look for subdirectories.
    next if $name =~ /^(\.|CVS)/;    # and...
    if (-d $name)                    # ... scan ONLY these.
    {
	$file = $name."/index.html";
	$out{$file}{name} = $name;
	$out{$file}{type} = "index";
	$out{$file}{file} = $name."/index.html";
	$out{$file}{sort} = "ac-macros";

	my $N = $name;
	$N =~ s/_/ /g;
	if ($N =~ /^[a-z]\w*$/)
	{
	    $out{$file}{sect} = "user";
	    $out{$file}{into} = $name;
	    $out{$file}{title} = $N."'s macros";
	    $out{$file}{chapter} = '<a href="http://sourceforge.net/users/'
		.$N.'">'.$N."</a>'s macros";
	}else{
	    $out{$file}{sect} = "gnus";
	    $out{$file}{sect} = "gnu" if $name =~ /Support/;
	    $out{$file}{into} = $name;
	    $out{$file}{title} = $N;
	    $out{$file}{chapter} = $N;
	}
	
	my @files = <$name/*.$m4>; 
	foreach $file (@files) 
	{
	    next if $file !~ /[.]$m4/;
	    my $F = $file; open F, "<$F" or next;
	    $file =~ s:[.]$m4$:.$htm:;
	    $out{$file}{text} = join ("",<F>); close F;
	    $out{$file}{orig} = $F;
	    $out{$file}{type} = "macro";
	    $out{$file}{into} = $name;
	    $out{$file}{category} = $name;
	}

	@files = <$name/*.txt>;
	foreach $file (@files)
	{
	    next if $file !~ /[.]txt/;
	    my $F = $file; open F, "<$F" or next;
	    $file =~ s:[.]txt$:.$htm:;
	    $out{$file}{text} = join ("",<F>); close F;
	    $out{$file}{orig} = $F;
	    $out{$file}{type} = "table";
	    $out{$file}{into} = $name;
	    $out{$file}{category} = $name;
	}
    }
}
closedir (DIR);

# ---------------------------------------------------------- MACRO PAGES ---
# just before this, we ran a routine to autodect m4 files in
# subdirectories. We assume here that no more m4 files will be
# coming in, and we put them to parse* and create* subroutines.

parse_spell_check ();

for $file (keys %out) # foreach $file of type "macro"
{
    next if not $out{$file}{type} eq "macro";
    print STDERR "." if not $o{quiet};
    parse_macro ($file) if not length $out{$file}{html}; # <-- MACRO2HTML
    create_html ($file) if not length $out{$file}{html}; # <-- MACRO2HTML
    parse_spell ($file) if not length $out{$file}{spelled};
}   print STDERR "\n" if not $o{quiet};

# ------------------------------------------------------- INFO TEXT PAGES ---
my %def;

$def{gnu}{webref} = "http://www.gnu.org/software/ac-archive/htmldoc/";
$def{gnu}{archive} = "GNU&nbsp;AC-Archive";
$def{gnus}{"C++_Support"}         = "index.html#CXX_SUPPORT";
$def{gnus}{"C_Support"}           = "index.html#C_SUPPORT";
$def{gnus}{"Cross_Compilation"}   = "index.html#CROSS_COMPILATION";
$def{gnus}{"Fortran_Support"}     = "index.html#FORTRAN_SUPPORT";
$def{gnus}{"Java_Support"}        = "index.html#JAVA_SUPPORT";
$def{gnus}{"Installed_Packages"}  = "index.html#INSTALLED_PACKAGES";
$def{gnus}{"Miscellaneous"}       = "index.html#MISC";
$def{html}{sfversion}="<!--VERSION-->*";

{
    for $file (keys %out) # foreach $file of type "table"
    {
	my $ini = "."; my $tion = ".";
	next if not $out{$file}{type} eq "table";
	for (split (/^/, $out{$file}{text} ))
	{
	    chomp; 
	    if (/^\#/ or /^\s*$/)
	    {
		next;
	    }elsif (/^(\w+)\s*(.*)$/)
	    { 
		print STDERR $ini,'/',$tion,'="',$def{$ini}{$tion},'"',$n;
		$ini = lc($1); $tion = $2;
		$tion =~ s/\s*$//;
		$def{$ini}{$tion} = "";
	    }else
	    { 
		$def{$ini}{$tion} .= "\n " if length $def{$ini}{$tion};
		s/^\s+//; s/\s+$//; s/~/\&nbsp\;/g; s:<//>:<br />:g;
		$def{$ini}{$tion} .= $_; 
	    }
		print STDERR $ini,'/',$tion,'="',$def{$ini}{$tion},'"',$n
		    if $ini ne ".";
	}       print STDERR "------------------------",$n;
    }
}

delete $def{"."};

$def{html}{logo} = $def{html}{sfnetimage};
$def{html}{sfversion}="<!--VERSION--><big><b>".$o{vers}."</b></big>" 
    if length $o{vers};

# ----------------------------------------------------------- SEARCH INDEX ---
# create an inverted index for all words in the input. Note
# that we trim down the output of `aspell` here that reached
# us through "sub parse_aspell" while reading the original
# text directly on input (it was saved in $out{$file}{spelled}).

my %search;

for $file (keys %out)
{
    next if not length $out{$file}{spelled};
    my $F = $out{$file}{spelled};
    next if not open F, "< $F";
    while (<F>)
    {
	chomp;
	next if not length;
	next if not /^(\w|-)/;
	next if /^[0-9]/;
	next if /^test\w/;
	next if /^(my|foo)-?(lib|pkg|config)/;
	next if /^\w*(dir|file|level|LEVEL)$/;
	next if /\b(CFLAGS|CXXFLAGS|LDFLAGS|CPPFLAGS|CVS|ACTION-IF)\b/;
	next if /^(linux|autoconf|automake|arg|func|ie|htmlhmm|fs|faq)$/i;
	next if /^(ac-archive|args|vm|todo|somedef|ahhm|subst|pc)$/i;
	next if /^(os|okay|nocheck|filesys|filesystem|cflags|bugfix)$/i;
	next if /^(autoheader|autoscan|api|a-z|a-z_|eg)$/i;
	next if m:^(w/o|\w+\'s)$:;

	$out{$file}{words} .= " " if length $out{$file}{words};
	$out{$file}{words} .= $_;
	$search{$_}{$file} = "";
    }
    close F;
}    

my $list = "search.html";
# $index{$list}{file} = "$list";
# $index{$list}{sect} = "virt";
$out{$list}{file} = "$list";
$out{$list}{name} = "search";
$out{$list}{type} = "index";
$out{$list}{sect} = "virt";
$out{$list}{navi} = "search";
$out{$list}{sort} = "ac-macros";
$out{$list}{title} = "search index";
$out{$list}{chapter} = "Inverted Search Index";
$out{$list}{body} = "<dl>".$n;
$out{$list}{info} = "";
for $name (sort keys %search)
{

    if ($name =~ /^[A-Z]/)
    {
	my $nm = lc($name);
	for $file (sort keys %{$search{$name}})
	{
	    $search{$nm}{$file} = "";
	}
	$search{$nm}{""} .= " ".$name;
	next;
    }

    if ($name =~ /^([a-z])/)
    {
	my $nm = uc($1);
	if ($out{$list}{info} !~ /\"\#$nm\"/)
	{
	    $out{$list}{info} .= $n.'<a href="#'.$nm.'">['.$nm.']</a>';
	    $out{$list}{body} .= '<a name="'.$nm.'"></a>'.$n;
	}
    }

    $out{$list}{body} .= "<dt>         ".$name.$search{$name}{""};
    $out{$list}{body} .= $n."</dt><dd>".$n;
    for $file (sort keys %{$search{$name}})
    {
	next if not length $file;
	$out{$list}{body} .= ' <a href="'.$file.'">'.$n;
	$file =~ s:[.]\w+$::;
	$out{$list}{body} .= '          '.$file.' </a> <br />'.$n;
    }
}

$out{$list}{body} .= "</dl>".$n;
$list = "";

# ------------------------------------------------------- MACRO CATEGORIES ---

for $file (keys %out) # foreach $file of type "macro"
{
    next if not $out{$file}{type} eq "macro";
    print STDERR "," if not $o{quiet};

    # category checks
    if (length $out{$file}{category})
    {
	my $also = $out{$file}{category};
	if (length $def{add}{$file})
	{   $also = "" if $also eq $out{$file}{into};
	    $also .= "," if length $also;
	    $also .= $def{add}{$file}; 
	}elsif (length $def{add}{ $out{$file}{name} })
	{   $also = "" if $also eq $out{$file}{into};
	    $also .= "," if length $also;
	    $also .= $def{add}{ $out{$file}{name} }; 
	}elsif (length $def{add}{ $out{$file}{orig} })
	{   $also = "" if $also eq $out{$file}{into};
	    $also .= "," if length $also;
	    $also .= $def{add}{ $out{$file}{orig} }; 
	}
	$out{$file}{obsoleted} = $also if $also =~ /obsoleted/;
	$also =~ s/\([^()]*\)//g;
	$also =~ s/,.*//;
	$also =~ s:^\s*::; $also =~ s:\s*$::; $also =~ s:\s+:_:g;
	$out{$file}{also} = $also if length $also;
    }

    if (length $out{$file}{obsoleted})
    {
	$out{$file}{also} = $out{$file}{into} if length $out{$file}{into};
	$out{$file}{into} = "obsoleted";
    }
}   print STDERR "\n" if not $o{quiet};

# ------------------------------------------------------- make %index / %sect
# fill the %index and %sect list,  these are inverted indexes 
# that are used later to speed up (ordered) processing

my %index;
for $file (keys %out)
{
    # fill %index with $type=="index" pages
    next if not $out{$file}{type} eq "index";
    $index{ $out{$file}{name} }{file} = $file;
    $index{ $out{$file}{name} }{sect} = $out{$file}{sect};
}

for $file (keys %out)
{
    next if not $out{$file}{type} eq "macro";
    $name = $out{$file}{into};
    $name = "local" if not length $name;
    if (not exists $index{$name})
    {
	$index{$name}{file} = "$name.html";
	$index{$name}{sect} = "virt";
	$out{"$name.html"}{name} = "$name";
	$out{"$name.html"}{file} = "$name.html";
	$out{"$name.html"}{type} = "index";
	$out{"$name.html"}{sect} = "virt";
	$out{"$name.html"}{sort} = "ac-macros";
	$out{"$name.html"}{title} = "$name";
	$out{"$name.html"}{chapter} = "$name macros";
    }
}


my %sect; # $sect{$ion}{$name} -> $file
for $name (keys %index)
{
    $sect{ $index{$name}{sect} }{ $name } = $index{$name}{file};
}

# ------------------------------------------------------ MAKE INDEX PAGES ---

for $list (sort keys %out)
{
    # which file goes listed on navi bar
    for $name (qw/index docus/)
    {	
	next if not $out{$list}{type} eq $name;
	$out{$list}{navi} = "$name" if not length $out{$list}{navi}; 
    }

    # check "index" pages
    next if not $out{$list}{type} eq "index";
    $out{$list}{body} .= "  <ul>\n";
    
    for $file (sort keys %out)
    {
	next if not $out{$file}{type} eq "macro";
	next if not $out{$file}{into} eq $out{$list}{name};
	my $m4file = $out{$file}{orig};
	my $htfile = $file;
	if ($list =~ m:/:)
	{
	    $m4file =~ s/.*\///;    # basename
	    $htfile =~ s/.*\///;    # basename
	}
	$m4file =~ s/\.\w+$//;  # chop ext
	$out{$list}{body} .= "    <li> <a href=\"$htfile\">$m4file</a>\n";
    }

    $out{$list}{body} .= "  </ul>\n";

    # and the "also" sections...
    my  $ion; my $last = "";
    for $ion (sort keys %sect) 
    {
	for $name (sort keys %{$sect{$ion}})
	{
	    next if $name eq $out{$list}{name};

	    for $file (sort keys %out)
	    {
		next if not $out{$file}{type} eq "macro";
		next if not $out{$file}{into} eq $name;
		next if not $out{$file}{also} eq $out{$list}{name};
		if ($last ne $name)
		{
		    $last = $name;
		    $out{$list}{body} .= "<h2>";
		    $out{$list}{body} .= $out{ $sect{$ion}{$name} }{chapter};
		    $out{$list}{body} .= ":"."</h2>\n  <ul>\n";
		}
		
		my $N = $file; $N =~ s/.*\///;
		my $htfile = (($list =~ m:/:)?"../":"").$file;
		$out{$list}{body} .= "    <li> <a href=\""
		    .$htfile."\">".$N."</a>\n";
	    }
	    $out{$list}{body} .= "  </ul>\n" if ($last eq $name);
	}
    }
    
    if (exists $def{info}{ $out{$list}{name} })
    {
	$name = $out{$list}{name};
	$out{$list}{info} .= '<p>&nbsp;</p> '.$n
	    .'<p align="center">'.$def{info}{$name}.'</p>'
	    .$n.'<p align="center">......................</p>'.$n;
    }
    if (exists $def{gnus}{ $out{$list}{name} })
    {
	$name = $out{$list}{name};
	$out{$list}{info} .= '<p>&nbsp;</p>' if not length $out{$list}{info};
	$out{$list}{info} .= $n
	    .'<p align="center">- mirrored -</p>'.$n.'<p class="justify">'.$n
	    .'The macros in the main section of this category are mirrored'.$n
	    .'from the'
	    .' <a href="'.$def{gnu}{webref}.'">'.$def{gnu}{archive}.'</a>'.$n
	    .' category of the same name.</p><p align="center">'.$n
	    .'<a href="'.$def{gnu}{webref}.$def{gnus}{$name}.'">';
	$name =~ s/_/&nbsp;/g;
	$out{$list}{info} .= $def{gnu}{archive};
	$out{$list}{info} .= "'s&nbsp; <b>".$name.'</b></a>'.$n.'</p>';
	$out{$list}{info} .= '<p>&nbsp;</p><p align="center"><small>'
	    .'<i>just the macros below are additional</i>'
	    .'</p>'.$n if length $last;
    }
    if (length $out{$list}{info} > 10)
    {
	$out{$list}{body} .= "<!--spacer-->".$n."<pre>";
	$out{$list}{body} .= " " x 50;
	$out{$list}{body} .= "</pre>".$n;
    }
}

$list = $topx; # the frontpage (only "gnus" sections are allowed here)
{   my  $ion = "gnus";
    for $ion (qw/gnu gnus/)
    {
	for $name (sort keys %{$sect{$ion}})
	{
	    $out{$list}{body} .= "<h2>"
		.$out{ $sect{$ion}{$name} }{chapter}.":"."</h2>\n";
	    $out{$list}{body} .= "  <ul>\n";
	    for $file (sort keys %out)
	    {
		next if not $out{$file}{type} eq "macro";
		# print "(".$out{$file}{into}.")";
		next if not $out{$file}{into} eq $name;
		my $N = $file; $N =~ s/.*\///;
		$out{$list}{body} .= "    <li> <a href=\""
		    .$file."\">".$N."</a>\n";
	    }
	    $out{$list}{body} .= "  </ul>\n";
	}
    }
}
# $out{$list}{name} = "$name";
$out{$list}{file} = "$list";
$out{$list}{type} = "index";
$out{$list}{navi} = "frontpage";
$out{$list}{sect} = "virt";
$out{$list}{sort} = "ac-macros";


# ------------------------------------------------------------ HTML OUTPUT ---

# add intro text for index pages
for $file (keys %out)
{
    next if not exists $def{intro}{$file};
    $out{$file}{info} = $def{intro}{$file};
}

my @indexlist;
{ # build sorted list of index pages
    my  $ion;
    for $ion (sort keys %sect)
    {
	for $name (sort keys %{$sect{$ion}})
	{
	    # print ":::",$name, " :: ", $sect{$ion}{$name},"\n";
	    push @indexlist, $sect{$ion}{$name};
	}
    }
}

my %cat;
{ # build sorted list of extra pages
    for $file (sort keys %out)
    {
	next if $out{$file}{type} ne "docus";
	$cat{ $out{$file}{sort} } = "1";
    }
}

# create navi-column ...................................................
# note that we create the navi-column's linkends as if the current page
# is in a first-level subdirectory, so that each of the sister pages
# is referenced via "../category/index.html"
my $navi = '<td width="145" bgcolor="#E0E0FF">'.$n; 
{ 
    $navi.="<!--logo:-->\n<center>\n"
	.$def{html}{logo}."</center>\n<!--logo/-->\n"
	.'<hr />'.$n
	."<!--site:-->\n".$def{html}{sfnetlinks}."<!--site/-->\n"
	.'<hr />'.$n;

    ####### my $back = ""; $back = "../" if $file =~ m:/:; ##############
    $navi.="<!--dirs:-->\n";
    $navi.='<a href="../'.$topx.'">Macro Index</a>'.$nn;
    foreach $name (@indexlist)
    {
	$navi.= '- <a href="../'.$name.'">'.$out{$name}{title}.'</a>'.$nn;
    }
    $navi.="<!--dirs/-->".$n.'<hr />'.$n;

    $navi.="<!--docs:-->".$n;
    my $category; 
    for $category (sort keys %cat)
    {
	next if -d $category or $category eq "ac-macros";
	print STDERR "<$category>";
	foreach $name (sort keys %out)
	{
	    next if not $out{$name}{type} eq "docus";
	    next if not $out{$name}{sort} eq $category;
	    $navi.='- ' if not $out{$name}{title} eq $out{$name}{category};
	    $navi.='<a href="../'.$name.'">'.$out{$name}{title}.'</a>';
	    $navi.=$nn;
	}
    }
    $navi.="<!--docs/-->".$n.'<hr />'.$n;

    $name = scalar localtime();
    $navi.='<small><i>generated...</i><br><center>'.$name.'</center></small>';
    $name =~ s/.*\s//;
    $navi.='<br><b><a href="copyright.html">(C) '.$name.'</a> ';
    $navi.=' <a href="mailto:guidod[@]gmx.de">guidod</a>';
    $navi.='</td>'.$n;
}
$out{"body.htm_"}{html} = $navi;

# -------------------------------------------------- ( GENERATE OUTPUT PAGES )
my $width100 =' width="100%"';
my $padding0=' cellspacing="0" cellpadding="0"';
my $valigntop=' valign="top"';

my $testfile = $o{test};
$testfile = "test.out" if length $testfile < 2;
unlink $testfile if length $o{test};

foreach $file (sort keys %out)                      # foreach output file...
{
    print STDERR $out{$file}{type}," ",$file, ".."; 
    
    my $F;
    if (length $o{test})
    {
	$F = $testfile;
	if (not open F, ">>$F")
	{
	    print "(could not open: $!)\n";
	    next; # OOPS
	}
    }else
    {
	$F = $file;
	if (not open F, ">$F")
	{
	    print "(could not open: $!)\n";
	    next; # OOPS
	}
    }
    
    if (length $out{$file}{html})
    { # pre-formatted html, e.g. macro2html result
	print F $out{$file}{html};
	close F; 
	print STDERR " -",length $out{$file}{html} unless $o{test}; 
	print STDERR $n;
	next; # DONE
    }
    
    print STDERR ".." unless $o{test}; # ____________
    # we assume a $out{$file}{body} to exist, we generate 
    # a navigation bar to the left and place appropriate
    # extra info according to $out{$file}{title/chapter/info}

    my $T = "";

    $T.="<html><head>";
    $T.="<title>".$def{short}{title}.' - '.$out{$file}{title}."</title>".$n;
    {
	$T.='<meta name="DC.Publisher"  content="'
	    .$def{short}{title}.'" />'.$n;
    }
    if ($out{$file}{type} eq "index")
    {	$T.='<meta name="DC.Type"       content="Collection"'.' />'.$n;  
	$T.='<meta name="DC.Title"      content="'
	    .$def{short}{title}.' Macro Overview : '
	    .$out{$file}{title}.'" />'.$n;
	$T.='<meta name="revisit-after" content="30 days"'.' />'.$n
	    unless $out{$file}{name} eq "search";
    }
    elsif ($out{$file}{type} eq "macro")
    {	$T.='<meta name="DC.Type"       content="Software"'.' />'.$n;  
	$T.='<meta name="DC.Title"      content="'
	    .$def{short}{title}.' Autoconf Extension '
	    .$out{$file}{title}.'" />'.$n;
	$T.='<meta name="DC.Relation"   content="'
	    .$def{short}{title}.' Macro Overview : '
	    .$out{$file}{into}.'" />'.$n;
	$T.='<meta name="...Relation"   content="'
	    .$def{short}{title}.' Macro Overview : '
	    .$out{$file}{also}.'" />'.$n if length $out{$file}{also};
    }
    elsif ($out{$file}{type} eq "docus")
    {	$T.='<meta name="DC.Type"       content="Text"'.' />'.$n;  }

    if (length $out{$file}{author})
    {
	$name = $out{$file}{author};
	$name =~ s/[^\w\s.-]/:/gs; $name =~ s/\w*:.*//gs; 
	$name =~ s/^\s+//gs;       $name =~ s/\s+$//gs;  $name =~ s/\s+/ /gs;
	$T.='<meta name="DC.Creator"    content="'.$name.'" />'.$n
	    if length $name;
    }
    if (length $out{$file}{orig})
    {
	my @A = stat $out{$file}{orig};
	@A = gmtime $A[9]; # $mtime
	$T.='<meta name="DC.Date"       content="'
	    .(1900+$A[5])."-".(1+$A[4])."-".(1+$A[3]).'" />'.$n;
	$T.='<meta name="DC.Source"     content="file:'
	    .$out{$file}{orig}.'" />'.$n;
    }
    if (length $out{$file}{words})
    {
	$T.='<meta name="words"         content="'
	    .$out{$file}{words}.'" />'.$n;
    }
    
    $T.="<style>\n".$def{html}{style}."</style>\n";
    $T.="</head><body>".$n.$n; # .................. </HEAD><BODY> ........

    if (not length $out{$file}{navi})
    {
	$T.= $out{$file}{body};
	$T.='</body></html>'.$n;
	print F $T; close F;
	print STDERR " +",length $T unless $o{test};
	print STDERR $n;
	next;
    }else{
	print STDERR "[",$out{$file}{navi},"]";
    }

    $T.="<table$width100><tr$valigntop>";
    {
	my $Q = $navi; # <td>...</td>
	$Q =~ s{^(-* *)(<a\s\Qhref=\"../$file\"\E.*)}{$1<b>$2</b>}mg;
##	$Q =~ s{<B>(.*\s\Qhref=\"../$file\"\E.*)</B>}{<b>$1</b>}mg;
	$Q =~ s{(\shref=\")\Q../\E}{$1}sg if ($file !~ m:/:);
###	$Q =~ s{<B>}{<I>}sg; $Q =~ s{</B>}{</I>}sg;
##	$Q =~ s{<B>}{   }sg; $Q =~ s{</B>}{    }sg;
	$T.= $Q;
    }
    $T.='<td>'.$n;          # ..................... </NAVI><PAGE> .........
    $T.='<!--name:-->'.$n; 
    $T.="<table$width100$padding0><tr>".'<td>'.$n;
    $T.="<h2>".$out{$file}{chapter}."</h2>"   if  length $out{$file}{chapter};
    $T.='<h3>The Autoconf Macro Archive</h3>' if !length $out{$file}{chapter};
    $T.= $n.'</td><td align="right">'.$n;
    $T.= $n.$def{html}{sfversion}.$n.$def{html}{sfbutton}.$n;
    $T.='</td>'.'</tr></table>'.$n;
    $T.='<!--name/-->'.$n;
    $T.="<table$width100><tr$valigntop>".'<td>' if length $out{$file}{info};
    $T.='<!--body:-->'.$n;
    $T.= $out{$file}{body};
    $T.='<!--body:-->'.$n;
    $T.="</td><td>\n<!--info:-->\n"          if length $out{$file}{info};
    $T.=$out{$file}{info}                    if length $out{$file}{info};
    $T.="\n<!--info/-->\n"                   if length $out{$file}{info};
    $T.="</td>"."</tr></table>"              if length $out{$file}{info};
    $T.='</td></tr></table>'.$n;
    $T.='</body></html>'.$n;

    print F $T; close F;
    print STDERR " *",length $T unless $o{test};
    print STDERR $n;
};
